<?php

namespace Fastrack\Property;

use Fastrack\Utilities\ModelUtilities;
use PlatoCreative\Imgix\Imgix;
use SilverStripe\Control\Controller;
use SilverStripe\Control\Director;
use SilverStripe\ORM\DataObject;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;

/**
 * @property string Name
 * @property string Description
 * @property int SortOrder
 * @property string SvgCode
 * @property string FontAwesomeClasses
 * @property string Key
 */
class Feature extends DataObject implements PermissionProvider
{
	private static string $table_name = 'feature';

	private static array $db = [
		'Title' => 'Varchar(255)',
		'Name' => 'Varchar(255)',
		'Description' => 'Text',
		'SvgCode' => 'Text',
		'FontAwesomeClasses' => 'Varchar(255)',
		'Key' => 'Varchar'
	];

	private static array $has_one = [
		'Image' => Imgix::class,
	];

	private static array $belongs_many_many = [
		'FeatureGroups' => FeatureGroup::class,
		'BookDirectFeatures' => BookDirectFeature::class,
	];

	private static array $owns = [
		'Image',
	];

	private static array $indexes = [
		'Key' => true,
	];

	private static array $translate = [
		'Title',
		'Name',
		'Description',
	];

	private static array $hidden_fields = [
		'Title',
		'SortOrder',
		'FeatureGroups',
		'BookDirectFeatures',
	];

	public function getCMSFields()
	{
		$fields = parent::getCMSFields();
		ModelUtilities::hideConfigFields($this, $fields);
		return $fields;
	}

	public function onBeforeWrite()
	{
		$this->Title = $this->Name;
		parent::onBeforeWrite();
	}

	public function CMSEditLink()
	{
		return Controller::join_links([
			$this->FeatureGroups()->first()->CMSEditLink(),
			'ItemEditForm',
			'field',
			'Features',
			'item',
			$this->ID,
		]);
	}

	public function WorkflowLink()
	{
		return Controller::join_links([Director::absoluteBaseURL(), $this->CMSEditLink()]);
	}

	public function providePermissions()
	{
		$permissions = [
			'MANAGE_FEATURE' => [
				'name' => 'Manage features',
				'category' => 'Features',
			],
		];
		$this->extend('updatePermissions', $permissions);
		return $permissions;
	}

	public function canView($member = null)
	{
		return Permission::check(['MANAGE_FEATURE']);
	}

	public function canEdit($member = null)
	{
		return Permission::check(['MANAGE_FEATURE']);
	}

	public function canDelete($member = null)
	{
		return Permission::check(['MANAGE_FEATURE']);
	}

	public function canCreate($member = null, $context = array())
	{
		return Permission::check(['MANAGE_FEATURE']);
	}

}
