<?php

namespace Fastrack\Property;

use SilverStripe\Forms\FieldList;
use SilverStripe\Forms\GridField\GridField;
use SilverStripe\Forms\GridField\GridFieldAddNewButton;
use SilverStripe\Forms\GridField\GridFieldConfig_RelationEditor;
use SilverStripe\Forms\GridField\GridFieldEditButton;
use SilverStripe\ORM\DataObject;
use SilverStripe\ORM\FieldType\DBBoolean;
use SilverStripe\ORM\FieldType\DBVarchar;
use SilverStripe\ORM\HasManyList;
use Symbiote\GridFieldExtensions\GridFieldAddExistingSearchButton;

/**
 * @property DBVarchar|string $Name
 * @property DBVarchar|string $Type
 * @property DBBoolean|bool $IsUAT
 * @property DBVarchar|string $Username
 * @property DBVarchar|string $AccessKey
 * @method HasManyList Properties()
 */
class PaymentGateway extends DataObject
{

	private static string $table_name = 'PaymentGateway';

	private static array $db = [
		'Name' => 'Varchar',
		'Type' => "Enum('Windcave')",
		'IsUAT' => 'Boolean',
		'Username' => 'Varchar',
		'AccessKey' => 'Varchar',
	];

	private static array $has_many = [
		'Properties' => Property::class,
	];

	public function getCMSFields(): FieldList
	{
		$fields = parent::getCMSFields();
		$gridConfig = GridFieldConfig_RelationEditor::create(100);
		$gridConfig->removeComponentsByType(GridFieldAddNewButton::class);
		$gridConfig->removeComponentsByType(GridFieldEditButton::class);
		$gridConfig->addComponent(new GridFieldAddExistingSearchButton());
		$properties = new GridField('Properties', 'Properties', $this->Properties(), $gridConfig);
		$fields->addFieldToTab('Root.Properties', $properties);
		$fields->removeByName('Name');
		return $fields;
	}

	protected function onBeforeWrite()
	{
		$this->Name = "{$this->Type} - {$this->Username}";
		parent::onBeforeWrite();
	}


	public function getUrl(): string
	{
		if ($this->Type === 'Windcave') {
			$mode = $this->IsUAT ? 'uat' : 'sec';
			return "https://{$mode}.windcave.com/api/v1/sessions";
		}
		return '';
	}

}
